from behave import *

def login(context, username, password):
    '''Will login the user with username and password. On default we we do
    a check on a successfull login'''
    logout(context)
    response = context.app.post('/auth/login',
        params={'login': username,
                'pass': password},
    )
    return response

def logout(context):
    'Logout the currently logged in user (if any)' 
    response = context.app.get('/auth/logout',
        params={}
    )
    return response

@given(u'a {role} user')
def step_impl(context, role):
    if role == "anonymous":
        assert (logout(context).status_int == 302)
    elif role == "admin":
        resp = login(context, "admin", "secret")
        assert (resp.status_int == 302)
    else:
        assert False



import uuid
import logging
import transaction
from pyramid.httpexceptions import HTTPFound, HTTPBadRequest
from pyramid.response import Response
from pyramid.view import view_config
import sqlalchemy as sa

from formbar.form import Form

from ringo.model.base import BaseList, BaseFactory
from ringo.model.user import User
from ringo.lib.renderer import ListRenderer, ConfirmDialogRenderer,\
DropdownFieldRenderer, ListingFieldRenderer
from ringo.lib.sql import invalidate_cache
from ringo.views import handle_history

log = logging.getLogger(__name__)


def _load_item(clazz, request):
    """Will load an item from the given clazz. The id of the item to
    load is taken from the request matchdict. If no item can be found an
    Exception is raised.

    :clazz: Class of the item to load
    :request: Current request having the id in the matchdict
    :returns: Loaded item

    """
    id = request.matchdict.get('id')
    factory = clazz.get_item_factory()
    item = factory.load(id, request.db)
    return item


def get_current_form_page(clazz, request):
    """Returns the id of the currently selected page. The currently
    selected page is saved in the session. If there is no saved value
    then the the first page is returned

    :clazz: The clazz for which the form is displayed
    :request: Current request
    :returns: id of the currently selected page. Default: 1
    """
    itemid = request.matchdict.get('id')
    item = clazz.__tablename__
    page = request.session.get('%s.%s.form.page' % (item, itemid))
    if page:
        return int(page)
    else:
        return 1


@view_config(route_name='set_current_form_page')
def set_current_form_page(request):
    """Will save the currently selected page of a form in the session.
    The request will have some attributes in the GET request which will
    config which page, of which item is currently shown. This function
    is used as a callback function within formbar.

    :request: Current request
    :returns: Response
    """
    page = request.GET.get('page')
    item = request.GET.get('item')
    itemid = request.GET.get('itemid')
    if page and item and itemid:
        #request.session['%s.form.page' % key] = page_id
        request.session['%s.%s.form.page' % (item, itemid)] = page
        request.session.save()
    return Response(body='OK', content_type='text/plain')


def handle_params(clazz, request):
    """Handles varios sytem GET params comming with the request
    Known params are:

     * backurl: A url which should be called instead of the default
       action after the next request succeeds. The backurl will be saved
       in the session and stays there until it is deleted on a
       successfull request. So take care to delete it to not mess up
       with the application logic.
    """
    backurl = request.GET.get('backurl')
    if backurl:
        request.session['%s.backurl' % clazz] = backurl
    request.session.save()


def handle_sorting(clazz, request):
    """Return a tuple of *fieldname* and *sortorder* (asc, desc). The
    sorting is determined in the follwoing order: First try to get the
    sorting from the current request (GET-Param). If there are no
    sorting params try to get the params saved in the session or if
    requested from a saved search. As last
    fallback use the default sorting for the table.
    """
    name = clazz.__tablename__

    # Default sorting options
    default_field = clazz.get_table_config().get_default_sort_column()
    default_order = 'asc'

    # Get sorting from the session. If there is no saved sorting use the
    # default value.
    field = request.session.get('%s.list.sort_field' % name, default_field)
    order = request.session.get('%s.list.sort_order' % name, default_order)

    # Get saved sorting from the the saved search.
    saved_search_id = request.params.get('saved')
    if saved_search_id:
        searches_dic = request.user.settings.get('searches', {})
        if searches_dic:
            search = searches_dic.get(name)
            if search:
                field, order = search.get(saved_search_id, [[], [], None])[1]

    # Get sorting from the request. If there is no sorting option in
    # the request then use the saved sorting options.
    field = request.GET.get('sort_field', field)
    order = request.GET.get('sort_order', order)

    # Save current sorting in the session
    if request.params.has_key('reset'):
        request.session['%s.list.sort_field' % name] = default_field
        request.session['%s.list.sort_order' % name] = default_order
    else:
        request.session['%s.list.sort_field' % name] = field
        request.session['%s.list.sort_order' % name] = order
    request.session.save()

    return field, order


def get_search(clazz, request):
    """Returns a list of tuples with the search word and the fieldname.
    The function will first look if there is already a saved search in
    the session for the overview of the given clazz. If there is no
    previous search the start with an empty search stack.  The following
    behavior differs depending if it is a POST or GET request:

    1. GET
    Return either an empty search stack or return the saved stack in the
    session.

    2. POST
    Get the new submitted search. If the search is not already on the
    stack, then push it.  If the search word is empty, then pop the last
    search from the stack.  Finally return the modified stack.

    Please note the this function will not save the modified search
    stack in the session! This should be done elsewhere. E.g Depending
    if the search was successfull.
    """
    name = clazz.__tablename__
    # Check if there is already a saved search in the session
    saved_search = request.session.get('%s.list.search' % name, [])

    if request.params.has_key('reset'):
        return []

    # If the request is not a equest from the search form then
    # abort here and return the saved search params if there are any.
    form_name = request.params.get('form')
    if form_name != "search":
        return saved_search

    saved_search_id = request.params.get('saved')
    if saved_search_id:
        searches_dic = request.user.settings.get('searches', {})
        if searches_dic:
            searches_dic_search = searches_dic.get(name)
            if searches_dic_search:
                return searches_dic_search.get(saved_search_id, [[], [], None])[0]
    elif request.params.has_key('save'):
        return saved_search
    elif request.params.has_key('delete'):
        return saved_search
    else:
        search = request.params.get('search')
        search_field = request.params.get('field')

    # If search is empty try to pop the last filter in the saved search
    if search == "" and len(saved_search) > 0:
        popped = saved_search.pop()
        log.debug('Popping %s from search stack' % repr(popped))

    # Iterate over the saved search. If the search is not already in the
    # stack push it.
    if search != "":
        found = False
        for x in saved_search:
            if search == x[0] and search_field == x[1]:
                found = True
                break
        if not found:
            log.debug('Adding search for "%s" in field "%s"' % (search, search_field))
            saved_search.append((search, search_field))
    return saved_search


def list_(clazz, request):
    # Important! Prevent any write access on the database for this
    # request. This is needed as transform would modify the items values
    # else.
    # TODO: Fix transform call to not modify the items in the
    # list.(None) <2013-08-12 21:26> 
    transaction.doom()
    handle_history(request)

    # If the user enters the overview page of an item we assume that the
    # user actually leaves any context where a former set backurl is
    # relevant anymore. So delete it.
    backurl = request.session.get('%s.backurl' % clazz)
    if backurl:
        # Redirect to the configured backurl.
        del request.session['%s.backurl' % clazz]
        request.session.save()

    rvalue = {}
    search = get_search(clazz, request)
    sorting = handle_sorting(clazz, request)
    listing = BaseList(clazz, request.db)
    listing.transform()
    listing.sort(sorting[0], sorting[1])
    listing.filter(search)
    # Only save the search if there are items
    if len(listing.items) > 0:
        request.session['%s.list.search' % clazz.__tablename__] = search
        if (request.params.get('form') == "search"):
            if request.params.has_key('save'):
                query_name = request.params.get('save')
                user = BaseFactory(User).load(request.user.id)
                searches_dic = user.settings.get('searches', {})
                searches_dic_search = searches_dic.get(clazz.__tablename__, {})

                # Check if there is already a search saved with the name
                found = False
                for xxx in searches_dic_search.values():
                    if xxx[1] == query_name:
                        found = True
                        break
                if not found:
                    searches_dic_search[str(uuid.uuid1())] = (search, sorting, query_name)
                searches_dic[clazz.__tablename__] = searches_dic_search
                user.settings.set('searches', searches_dic)
                request.db.flush()
            elif request.params.has_key('delete'):
                query_key = request.params.get('delete')
                user = BaseFactory(User).load(request.user.id)
                searches_dic = user.settings.get('searches', {})
                searches_dic_search = searches_dic.get(clazz.__tablename__, {})
                try:
                    del searches_dic_search[query_key]
                except:
                    pass
                searches_dic[clazz.__tablename__] = searches_dic_search
                user.settings.set('searches', searches_dic)
                request.db.flush()
        request.session.save()


    renderer = ListRenderer(listing)
    rendered_page = renderer.render(request)
    rvalue['clazz'] = clazz
    rvalue['listing'] = rendered_page
    return rvalue


def create_(clazz, request, callback=None, renderers={}):
    """Base view to create a new item of type clazz. This view will
    render a create form to create new items. It the user submits the
    data (POST) that the data will be validated and the new item will be
    saved to the database. Finally after saving on the POST-request the
    optional callback will be called.

    :clazz: Class of items which will be created.
    :request: The current request
    :callback: A callback function [function(request, item)] which
    returns the item again.
    :renderers: A optional dictionary of custom renderers which are
    provided to the form to render specific formelements.
    :returns: Dictionary with the following keys 'clazz', 'item', 'form'
    """
    handle_history(request)
    handle_params(clazz, request)
    _ = request.translate
    rvalue = {}
    # Add ringo specific renderers
    if not "dropdown" in renderers:
        renderers["dropdown"] = DropdownFieldRenderer
    if not "listing" in renderers:
        renderers["listing"] = ListingFieldRenderer
    factory = clazz.get_item_factory()
    item = factory.create(request.user)
    form = Form(item.get_form_config('create'), item, request.db, translate=_,
                renderers=renderers,
                change_page_callback={'url': 'set_current_form_page',
                                      'item': clazz.__tablename__,
                                      'itemid': None},
                request=request)
    if request.POST:
        item_label = clazz.get_item_modul().get_label()
        mapping = {'item_type': item_label}
        if form.validate(request.params):
            sitem = form.save()
            msg = _('Created new ${item_type} successfull.',
                    mapping=mapping)
            log.info(msg)
            request.session.flash(msg, 'success')
            # flush the session to make the new id in the element
            # presistent.
            request.db.flush()
            route_name = sitem.get_action_routename('update')
            url = request.route_url(route_name, id=sitem.id)
            if callback:
                sitem = callback(request, sitem)
            # Invalidate cache
            invalidate_cache()
            # Handle redirect after success.
            backurl = request.session.get('%s.backurl' % clazz)
            if backurl:
                # Redirect to the configured backurl.
                del request.session['%s.backurl' % clazz]
                request.session.save()
                return HTTPFound(location=backurl)
            else:
                # Redirect to the update view.
                return HTTPFound(location=url)
        else:
            msg = _('Error on validation the data'
                    ' for new ${item_type}', mapping=mapping)
            request.session.flash(msg, 'error')
    rvalue['clazz'] = clazz
    rvalue['item'] = item
    rvalue['form'] = form.render(page=get_current_form_page(clazz, request))
    return rvalue


def update_(clazz, request, callback=None, renderers={}):
    handle_history(request)
    handle_params(clazz, request)
    _ = request.translate
    rvalue = {}
    # Add ringo specific renderers
    if not "dropdown" in renderers:
        renderers["dropdown"] = DropdownFieldRenderer
    if not "listing" in renderers:
        renderers["listing"] = ListingFieldRenderer
    id = request.matchdict.get('id')
    factory = clazz.get_item_factory()

    # Load the item return 400 if the item can not be found.
    try:
        item = factory.load(id, request.db)
    except sa.orm.exc.NoResultFound:
        raise HTTPBadRequest()

    form = Form(item.get_form_config('update'), item, request.db, translate=_,
                renderers=renderers,
                change_page_callback={'url': 'set_current_form_page',
                                      'item': clazz.__tablename__,
                                      'itemid': id},
                request=request)
    if request.POST:
        item_label = clazz.get_item_modul().get_label()
        mapping = {'item_type': item_label, 'item': item}
        if form.validate(request.params):
            form.save()
            msg = _('Edited ${item_type} "${item}" successfull.',
                    mapping=mapping)
            log.info(msg)
            request.session.flash(msg, 'success')
            route_name = item.get_action_routename('update')
            url = request.route_url(route_name, id=item.id)
            if callback:
                item = callback(request, item)
            # Invalidate cache
            invalidate_cache()
            # Handle redirect after success.
            backurl = request.session.get('%s.backurl' % clazz)
            if backurl:
                # Redirect to the configured backurl.
                del request.session['%s.backurl' % clazz]
                request.session.save()
                return HTTPFound(location=backurl)
            else:
                # Redirect to the update view.
                return HTTPFound(location=url)
        else:
            msg = _('Error on validation the data for '
                    '${item_type} "${item}".', mapping=mapping)
            log.info(msg)
            request.session.flash(msg, 'error')

    rvalue['clazz'] = clazz
    rvalue['item'] = item
    rvalue['form'] = form.render(page=get_current_form_page(clazz, request))
    return rvalue


def read_(clazz, request, renderers={}):
    handle_history(request)
    handle_params(clazz, request)
    _ = request.translate
    rvalue = {}
    # Add ringo specific renderers
    if not "dropdown" in renderers:
        renderers["dropdown"] = DropdownFieldRenderer
    if not "listing" in renderers:
        renderers["listing"] = ListingFieldRenderer
    id = request.matchdict.get('id')
    factory = clazz.get_item_factory()

    # Load the item return 400 if the item can not be found.
    try:
        item = factory.load(id, request.db)
    except sa.orm.exc.NoResultFound:
        raise HTTPBadRequest()

    form = Form(item.get_form_config('read'), item, request.db, translate=_,
                renderers=renderers,
                change_page_callback={'url': 'set_current_form_page',
                                      'item': clazz.__tablename__,
                                      'itemid': id},
                request=request)
    rvalue['clazz'] = clazz
    rvalue['item'] = item
    rvalue['form'] = form.render(page=get_current_form_page(clazz, request))
    return rvalue


def delete_(clazz, request):
    handle_history(request)
    handle_params(clazz, request)
    _ = request.translate
    rvalue = {}
    id = request.matchdict.get('id')
    factory = clazz.get_item_factory()

    # Load the item return 400 if the item can not be found.
    try:
        item = factory.load(id, request.db)
    except sa.orm.exc.NoResultFound:
        raise HTTPBadRequest()

    if request.method == 'POST' and confirmed(request):
        request.db.delete(item)
        route_name = clazz.get_action_routename('list')
        url = request.route_url(route_name)
        item_label = clazz.get_item_modul().get_label()
        mapping = {'item_type': item_label, 'item': item}
        msg = _('Deleted ${item_type} "${item}" successfull.', mapping=mapping)
        log.info(msg)
        request.session.flash(msg, 'success')
        # Invalidate cache
        invalidate_cache()
        # Handle redirect after success.
        backurl = request.session.get('%s.backurl' % clazz)
        if backurl:
            # Redirect to the configured backurl.
            del request.session['%s.backurl' % clazz]
            request.session.save()
            return HTTPFound(location=backurl)
        else:
            # Redirect to the update view.
            return HTTPFound(location=url)
    else:
        # FIXME: Get the ActionItem here and provide this in the Dialog to get
        # the translation working (torsten) <2013-07-10 09:32> 
        renderer = ConfirmDialogRenderer(request, item, 'delete')
        rvalue['dialog'] = renderer.render()
        rvalue['clazz'] = clazz
        rvalue['item'] = item
        return rvalue


def confirmed(request):
    """Returns True id the request is confirmed"""
    return request.params.get('confirmed') == "1"



import unittest
from sqlalchemy.orm.exc import NoResultFound
from ringo.model.user import User
from ringo.tests import BaseFunctionalTest

class TestUser(BaseFunctionalTest):

    def test_wrong_url(self):
        self.login('admin', 'secret')
        self.app.get('/users/listtest', status=404)

    def test_list(self):
        self.login('admin', 'secret')
        self.app.get('/users/list', status=200)

    def test_read(self):
        self.login('admin', 'secret')
        self.app.get('/users/read/1', status=200)

    def test_read_unknown(self):
        self.login('admin', 'secret')
        try:
            self.app.get('/users/read/1234')
        except NoResultFound:
            return True
        # Must fail!
        self.assertEqual(0,1)

    def test_create(self):
        self.login('admin', 'secret')
        self.app.get('/users/create', status=200)

    def test_create_save_fail(self):
        self.login('admin', 'secret')
        # fail because password missmatch!
        values = {
            'login': 'testuser',
            'activation_token': '',
            'password': '12345678',
            'retype_password': '123456789'
        }
        self.app.post('/users/create', values, status=200)

    def test_create_save_success(self):
        self.login('admin', 'secret')
        # fail because password missmatch!
        values = {
            'login': 'testuser',
            'activation_token': '',
            'password': '12345678',
            'retype_password': '12345678'
        }
        self.app.post('/users/create', values, status=302)

    def test_edit(self):
        self.login('admin', 'secret')
        self.app.get('/users/update/1', status=200)

    def test_edit_save_ok(self):
        self.login('admin', 'secret')
        values = {
            'login': 'admin',
            'groups': '1',
            'gid': '',
            'activated': 'True',
            'activation_token': ''
        }
        self.app.post('/users/update/1', values, status=302)

    def test_edit_save_fail(self):
        self.login('admin', 'secret')
        values = {
            'login': '',
            'groups': '1',
            'gid': '',
            'activated': 'True',
            'activation_token': ''
        }
        self.app.post('/users/update/1', values, status=200)

    def test_delete(self):
        last_id = self.get_max_id(User)
        self.login('admin', 'secret')
        self.app.get('/users/delete/%s' % last_id, status=200)

    def test_delete_confirmed(self):
        last_id = self.get_max_id(User)
        self.login('admin', 'secret')
        values = {'confirmed': '1'}
        self.app.post('/users/delete/%s' % last_id, values, status=302)

    def test_delete_fail(self):
        last_id = self.get_max_id(User)
        self.login('admin', 'secret')
        values = {'confirmed': '0'}
        self.app.post('/users/delete/%s' % last_id, values, status=200)

if __name__ == '__main__':
    unittest.main()



import logging
import transaction
import re
import datetime
from operator import itemgetter
from formbar.config import Config, load
from sqlalchemy.orm import joinedload, ColumnProperty, class_mapper
from ringo.lib.helpers import get_path_to_form_config
from ringo.lib.sql import DBSession, regions
from ringo.lib.sql.query import FromCache, set_relation_caching

log = logging.getLogger(__name__)


class BaseItem(object):

    _modul_id = None
    """Configure a list of relations which are configured to be joined
    in the query"""
    _sql_joined_relations = []
    # TODO: Defining relations which should be cached seems to be
    # ignored for lazy load. So the only way of caching those
    # relations too is to put them in the SQL-JOIN and to a cached
    # query. This way all relations are cached to.
    """Configure a list of relations which are configured to be
    cached"""
    _sql_cached_realtions = []

    def __str__(self):
        return self.__unicode__()

    def __getitem__(self, name):
        return getattr(self, name)

    def __json__(self, request):
        rvalue = {}
        for col in self.get_columns():
            value = getattr(self, col)
            # FIXME: If value is type date it can not be serialized.
            if isinstance(value, datetime.date):
                value = str(value)
            rvalue[col] = value
        return rvalue

    @classmethod
    def get_columns(cls):
        return [prop.key for prop in class_mapper(cls).iterate_properties
            if isinstance(prop, ColumnProperty)]

    @classmethod
    def get_item_factory(cls):
        return BaseFactory(cls)

    @classmethod
    def get_item_list(cls, db):
        return BaseList(cls, db)

    @classmethod
    def get_item_modul(cls):
        from ringo.model.modul import ModulItem
        factory = BaseFactory(ModulItem)
        return factory.load(cls._modul_id)

    @classmethod
    def get_table_config(cls, tablename=None):
        from ringo.lib.renderer import TableConfig
        return TableConfig(cls, tablename)

    @classmethod
    def get_form_config(cls, formname):
        """Return the Configuration for a given form. The configuration
        tried to be loaded from the application first. If this fails it
        tries to load it from the ringo application."""
        cfile = "%s.xml" % cls.__tablename__
        try:
            loaded_config = load(get_path_to_form_config(cfile))
        except IOError:
            loaded_config = load(get_path_to_form_config(cfile, 'ringo'))
        config = Config(loaded_config)
        return config.get_form(formname)

    @classmethod
    def get_action_routename(cls, action, prefix=None):
        routename = "%s-%s" % (cls.__tablename__, action)
        if prefix:
            return "%s-%s" % (prefix, routename)
        return routename


class BaseList(object):
    def __init__(self, clazz, db, cache=""):
        """
        A List object of. A list can be filterd, and sorted.

        :clazz: Class of items which will be loaded.
        :db: DB session to load the item
        :cache: Name of the cache region. If empty then no caching is
        done.
        """
        self.clazz = clazz
        # TODO: Check which is the best loading strategy here for large
        # collections. Tests with 100k datasets rendering only 100 shows
        # that the usual lazyload method seems to be the fastest which is
        # not what if have been expected.
        #self.items = db.query(clazz).options(joinedload('*')).all()

        q = db.query(self.clazz)
        if cache in regions.keys():
            q = set_relation_caching(q, self.clazz, cache)
            q = q.options(FromCache(cache))
        for relation in self.clazz._sql_cached_realtions:
            q = q.options(joinedload(relation))

        self.items = q.all()
        self.search_filter = []

    def __json__(self, request):
        return self.items

    def transform(self, form_id="create"):
        """Will transform the values of the items in the list. This will
        apply all configured expandation of values or other
        transformations like custom renderings etc. If there is any
        expandation configured for a column in the table then the form
        configuration for the clazz is loaded to get the expanded value
        (options for selection).

        Please note that calling this function is only allowed for a
        "doomed" db request (transaction) as the function modifies the
        data of the items which would be written to the database
        otherwise.

        :form_id: The id of the form configuration which is used for
        expandation.
        """

        if transaction.isDoomed() is False:
            raise Exception("Calling transform on a item list is only "
                            "allowed if the current request is doomed to "
                            "prevent accidential data modifications.")

        # Check if there are any expandations configured
        expand = []
        table_config = self.clazz.get_table_config()
        for col in table_config.get_columns():
            if col.get('expand'):
                expand.append(col.get('name'))
        if len(expand) == 0:
            # Nothing to expand here. So break here.
            return
        else:
            # Load form configuration used for the mapping
            form_config = self.clazz.get_form_config(form_id)

        # Iterate over over all fields and replace to raw_value with the
        # expanded value from the form configuration.
        for item in self.items:
            for field in expand:
                raw_value = getattr(item, field)
                field_config = form_config.get_field(field)
                for option in field_config.options:
                    if str(raw_value) == str(option[1]):
                        setattr(item, field, option[0])

    def sort(self, field, order):
        sorted_items = sorted(self.items, key=itemgetter(field))
        if order == "desc":
            sorted_items.reverse()
        self.items = sorted_items

    def filter(self, filter_stack):
        """This function will filter the list of items by only leaving
        those items in the list which match all search criterias in the
        filter stack. The list will get reduced with every iteration on
        the filter stack.

        For each filter in the stack The search word will be compiled to
        the regular expression. Then all items are iterated.  For each
        item the function will try to match the value of either all, or
        from the configued search field with the regular expression. If
        the value matches, then the item is kept in the list.
        """
        self.search_filter = filter_stack
        for search, search_field in filter_stack:
            # Build a regular expression
            re_expr = re.compile(search)
            filtered_items = []
            log.debug('Filtering "%s" in "%s" on %s items'
                      % (search, search_field, len(self.items)))
            if search_field != "":
                fields = [search_field]
            else:
                table_config = self.clazz.get_table_config()
                fields = [field.get('name') for field in table_config.get_columns()]
            for item in self.items:
                for field in fields:
                    value = getattr(item, field)
                    if isinstance(value, list):
                        value = ", ".join([unicode(x) for x in value])
                    else:
                        value = unicode(value)
                    if re_expr.search(value):
                        filtered_items.append(item)
                        break
            self.items = filtered_items


class BaseFactory(object):

    def __init__(self, clazz):
        """Factory to create of load instances of the clazz.

        :clazz: The clazz of which new items will be created

        """
        self._clazz = clazz

    def create(self, user):
        """Will create a new instance of clazz. The instance is it is not saved
        persistent at this moment. The method will also take care of
        setting the correct ownership.

        :user: User instance will own the new created item
        :returns: Instance of clazz

        """
        item = self._clazz()
        # Try to set the ownership of the entry if the item provides the
        # fields.
        if (hasattr(item, 'uid')
        and hasattr(item, 'gid')
        and user is not None):
            item.uid = user.id
            item.gid = user.gid
        return item

    def load(self, id, db=None, cache=""):
        """Loads the item with id from the database and returns it.

        :id: ID of the item to be loaded
        :db: DB session to load the item
        :cache: Name of the cache region. If empty then no caching is
        done.
        :returns: Instance of clazz

        """
        if db is None:
            db = DBSession
        q = db.query(self._clazz)
        if cache in regions.keys():
            q = set_relation_caching(q, self._clazz, cache)
            q = q.options(FromCache(cache))
        for relation in self._clazz._sql_joined_relations:
            q = q.options(joinedload(relation))
        return q.filter(self._clazz.id == id).one()



from behave import *

@when(u'opens the login page')
def step_impl(context):
    response = context.app.get('/auth/login')
    assert (response.status_int == 200)

@when(u'submits valid login data')
def step_impl(context):
    response = context.app.post('/auth/login',
        params={'login': 'admin',
                'pass': 'secret'},
    )
    context.resp = response

@when(u'submits invalid login data')
def step_impl(context):
    response = context.app.post('/auth/login',
        params={'login': 'admin',
                'pass': 'wrongpassword'},
    )
    context.resp = response

@when(u'opens the registration page')
def step_impl(context):
    response = context.app.get('/auth/register_user')
    assert (response.status_int == 200)

@when(u'submits valid registration data')
def step_impl(context):
    response = context.app.post('/auth/register_user',
        params={'login': 'testuser',
                'pass': 'mypass',
                'pass2': 'mypass',
                'email': 'testuser@example.com',
        },
    )
    context.resp = response

@when(u'opens the password reminder page')
def step_impl(context):
    response = context.app.get('/auth/forgot_password')
    assert (response.status_int == 200)

@when(u'submits valid userdata data')
def step_impl(context):
    response = context.app.post('/auth/forgot_password',
        params={'login': 'admin'},
    )
    context.resp = response

@when(u'submits invalid userdata data')
def step_impl(context):
    response = context.app.post('/auth/forgot_password',
        params={'login': 'unknowuser'},
    )
    context.resp = response




import logging
import hashlib
import uuid
import string
import random
from datetime import datetime

from pyramid.security import unauthenticated_userid,\
    has_permission as has_permission_
from pyramid.authentication import AuthTktAuthenticationPolicy
from pyramid.authorization import ACLAuthorizationPolicy

from sqlalchemy.orm.exc import NoResultFound

from ringo.lib.sql import DBSession
from ringo.model.user import User, PasswordResetRequest

log = logging.getLogger(__name__)


def password_generator(size=8, chars=string.ascii_uppercase + string.digits):
    return ''.join(random.choice(chars) for x in range(size))


def setup_ringo_security(config):
    authn_policy = AuthTktAuthenticationPolicy('seekrit',
                                               hashalg='sha512',
                                               callback=get_principals)
    authz_policy = ACLAuthorizationPolicy()
    config.set_authentication_policy(authn_policy)
    config.set_authorization_policy(authz_policy)
    # Make the user object available as attribute "user" in the request.
    # See http://docs.pylonsproject.org/projects/ \
    #            pyramid_cookbook/en/latest/auth/ \
    #            user_object.html
    config.set_request_property(get_user, 'user', reify=True)


def get_user(request):
    userid = unauthenticated_userid(request)
    if userid is not None:
        return _load_user(userid, request)
    return None


def has_permission(permission, context, request):
    return has_permission_(permission, context, request)


def get_principals(userid, request):
    """Returns a list of pricipals for the user with userid for the
    given request.

    Principals are basically strings naming the groups or roles the user have.
    Example: role:admin or group:users are typical principals.

    :userid: id of the user
    :request: current request
    :returns: list with pricipals

    """
    user = _load_user(userid, request)
    principals = []
    if user:
        # Add roles the user have
        for role in get_roles(user):
            principals.append('role:%s' % role.name)
        # Add groups the user member of
        for group in user.groups:
            principals.append('group:%s' % group.name)
        # Finally add the user itself
        principals.append('uid:%s' % user.id)


    log.debug('Principals for user "%s": %s' % (user.login, principals))
    return principals

# ROLES
#######


def has_role(user, role):
    """Return True if the user has the given role. Else False"
    :user: User instance
    :returns: True or False
    """

    roles = [r.name for r in get_roles(user)]
    return role in roles


def get_roles(user):
    """Returns a list of roles the user has. The list contains
    `Role` object and are collected by loading roles directly
    attached to the user plus roles attached to the groups the user
    is member of

    :user: User instance
    :returns: List of `Role` instances

    """
    tmp_roles = {}

    # Add roles directly attached to the user.
    for role in user.roles:
        if role.name not in tmp_roles:
            tmp_roles[role.name] = role

    # Add roles directly attached to the user.
    for group in user.groups:
        for role in group.roles:
            if role.name not in tmp_roles:
                tmp_roles[role.name] = role

    return list(tmp_roles.values())

# GROUPS
########


def has_group(user, group):
    """Return True if the user is in the the given group. Else False"
    :user: User instance
    :returns: True or False
    """

    groups = [g.name for g in user.groups]
    return group in groups

# Helpers
#########


def _load_user(userid, request):
    try:
        factory = User.get_item_factory()
        return factory.load(userid)
    except NoResultFound:
        return None


def activate_user(token, db):
    """Will check if confirmation token is valid by searching a user
    with this activation token. If a user is found the activated flag
    will be set to true.

    :token: confirmation token
    :db: db connection
    :returns: user
    """
    try:
        user = db.query(User).filter_by(activation_token=token).one()
        user.activated = True
        user.activation_token = None
        log.info('User %s activated' % user)
        return user
    except NoResultFound:
        log.warning('User activation failed for token %s'
                    % (token))
        return None


def password_reset(token, db):
    """Will check if there is a password reset token is valid. The
    function will reset the password of the user. The new generated
    password is returned.

    :token: password reset token
    :db: db connection
    :returns: tupe of user and password
    """
    try:
        token = db.query(PasswordResetRequest).filter_by(token=token).one()
        # Check that the token is not outdated
        td = datetime.now() - token.created
        if td.days <= 1:
            user = token.user
            password = password_generator()
            md5_pw = hashlib.md5()
            md5_pw.update(password)
            md5_pw = md5_pw.hexdigest()
            user.password = md5_pw
            log.info('Password reset success for user %s' % user)
            # delete all old password request token
            for old_token in user.reset_tokens:
                db.delete(old_token)
            return user, password
        else:
            log.warning('Password reset failed for token %s (outdated)'
                        % token)
            return None, None
    except NoResultFound:
        log.warning('Password reset failed for token %s' % token)
        return None, None


def request_password_reset(username, db):
    """Will check if there is a user with the given username and creates
    a new entry in the password_reset table with a new token. The new
    token is than returned. If there is no user with the username then
    do nothing and return None.

    :username: username
    :db: db connection
    :returns: User

    """
    user = None
    try:
        user = db.query(User).filter_by(login=username).one()
        log.info('Password reset request for user %s' % user)
        reset_token = PasswordResetRequest(str(uuid.uuid4()))
        user.reset_tokens.append(reset_token)
    except NoResultFound:
        log.warning('Password reset request for non existing user %s'
                    % username)
    return user


def login(username, password):
    """Returns a `User` instance if the login does not fail with the
    given login and password.

    :username: username as String
    :password: password as SHA1 encrypted string
    :returns: `User` instance if login is OK else None.

    """
    md5_pw = hashlib.md5()
    md5_pw.update(password or "")
    md5_pw = md5_pw.hexdigest()
    log.debug("Login user '%s' with pw '%s'" % (username, password))
    try:
        user = DBSession.query(User).filter_by(login=username,
                                               password=md5_pw).one()
        if user.activated:
            log.info("Login successfull '%s'" % (username))
            return user
        log.info("Login failed for user '%s'. "
                 "Reason: Not activated" % username)
    except NoResultFound:
        log.info("Login failed for user '%s'. "
                 "Reason: Username or Password wrong" % username)
    return None



import unittest
from sqlalchemy.orm.exc import NoResultFound
from ringo.model.user import Role
from ringo.tests import BaseFunctionalTest


class TestRole(BaseFunctionalTest):

    def test_wrong_url(self):
        self.login('admin', 'secret')
        self.app.get('/roles/listtest', status=404)

    def test_list(self):
        self.login('admin', 'secret')
        self.app.get('/roles/list', status=200)

    def test_sort(self):
        self.login('admin', 'secret')
        values = {
            'sort_field': 'name',
            'sort_order': 'desc'
        }
        self.app.get('/roles/list', values, status=200)

    def test_read(self):
        self.login('admin', 'secret')
        self.app.get('/roles/read/1', status=200)

    def test_read_unknown(self):
        self.login('admin', 'secret')
        try:
            self.app.get('/roles/read/1234')
        except NoResultFound:
            return True
        # Must fail!
        self.assertEqual(0,1)

    def test_create(self):
        self.login('admin', 'secret')
        self.app.get('/roles/create', status=200)

    def test_create_save_fail(self):
        self.login('admin', 'secret')
        # fail because name must not be none!
        values = {
            'name': ''
        }
        self.app.post('/roles/create', values, status=200)

    def test_create_save_success(self):
        self.login('admin', 'secret')
        values = {
            'name': 'testrole',
            'admin': 'True'
        }
        self.app.post('/roles/create', values, status=302)

    def test_edit(self):
        self.login('admin', 'secret')
        self.app.get('/roles/update/1', status=200)

    def test_edit_save_ok(self):
        self.login('admin', 'secret')
        values = {
            'name': 'admin',
            'admin': 'True'
        }
        self.app.post('/roles/update/1', values, status=302)

    def test_edit_save_fail(self):
        self.login('admin', 'secret')
        # Missing value
        values = {
            'name': ''
        }
        self.app.post('/roles/update/1', values, status=200)

    def test_delete(self):
        last_id = self.get_max_id(Role)
        self.login('admin', 'secret')
        self.app.get('/roles/delete/%s' % last_id, status=200)

    def test_delete_confirmed(self):
        last_id = self.get_max_id(Role)
        self.login('admin', 'secret')
        values = {'confirmed': '1'}
        self.app.post('/roles/delete/%s' % last_id, values, status=302)

    def test_delete_fail(self):
        last_id = self.get_max_id(Role)
        self.login('admin', 'secret')
        values = {'confirmed': '0'}
        self.app.post('/roles/delete/%s' % last_id, values, status=200)

if __name__ == '__main__':
    unittest.main()



import os
import pkg_resources
from pyramid.threadlocal import get_current_registry
from formbar.helpers import get_css, get_js


def dynamic_import(cl):
    d = cl.rfind(".")
    classname = cl[d+1:len(cl)]
    m = __import__(cl[0:d], globals(), locals(), [classname])
    return getattr(m, classname)


def get_ringo_version():
    return pkg_resources.get_distribution('ringo').version


def get_app_name():
    registry = get_current_registry()
    return registry.__name__


def get_app_version():
    return pkg_resources.get_distribution(get_app_name()).version


def get_app_location(name=None):
    if not name:
        name = get_app_name()
    return pkg_resources.get_distribution(name).location


def get_app_title():
    registry = get_current_registry()
    settings = registry.settings
    return settings['app.title']


def get_action_url(request, item, action):
    """Return an URL object for the given item and action. If the item
    is an instance of object then we assume that we want to get the URL
    for the specific item. So we add the ID of the instance to the url.
    If the item is the class, then no ID is added (Create actions e.g.)

    :item: loaded instance or class of an item
    :action: string of the action
    :returns: URL instance
    """
    base_name = item.__tablename__
    route_name = "%s-%s" % (base_name, action)
    if isinstance(item, object):
        return request.route_url(route_name, id=item.id)
    return request.route_url(route_name)


def get_path_to(location, app=None):
    """Will return the full pathname the given file name with in the
    path. path is relativ to the application package (pkg_ressource
    location + ressource name). You can define a alternative
    application."""
    if app:
        app_name = app
    else:
        app_name = get_app_name()
    base_path = os.path.join(get_app_location(app_name), app_name)
    return os.path.join(base_path, location)


def get_path_to_form_config(filename, app=None):
    """Returns the path the the given form configuration. The file name
    should be realtive to the default location for the configurations.

    :file: filename
    :returns: Absolute path to the configuration file

    """
    location = "views/forms"
    return get_path_to(os.path.join(location, filename), app)


def get_path_to_overview_config(filename, app=None):
    """Returns the path the the given overview configuration. The file name
    should be realtive to the default location for the configurations.

    :file: filename
    :returns: Absolute path to the configuration file

    """
    location = "views/tables"
    return get_path_to(os.path.join(location, filename), app)


def get_saved_searches(request, name):
    """Returns a dictionary with the saved searches for the named overview.
    :request: Current request
    :name: Name of the overview
    :returns: Dictionary with the saved seaerches.
    """
    if not request.user:
        return {}
    searches_dic = request.user.settings.get('searches', {})
    if searches_dic:
        searches_dic_search = searches_dic.get(name, {})
        return searches_dic_search
    return {}


def get_modules(request, display):
    # TODO: Fix imports here. Seems to be circular imports.
    from ringo.model.modul import ModulItem
    from ringo.resources import get_resource_factory
    from ringo.lib.security import has_permission
    listing = ModulItem.get_item_list(request.db)
    user_moduls = []
    for item in listing.items:
        # Only show the modul if it matches the desired display location
        # and if the modul has an "list" action which usually is used as
        # entry point into a modul.
        if (item.display == display and item.has_action('list')):
            # Build a ressource and to be able to check the current user
            # permissions against it.
            clazz = dynamic_import(item.clazzpath)
            factory = get_resource_factory(clazz)
            resource = factory(request)
            if has_permission('list', resource, request):
                user_moduls.append(item)
    return user_moduls

def get_formbar_css():
    return get_css()

def get_formbar_js():
    return get_js()
    return get_css()



import hashlib
import uuid

from pyramid.security import remember, forget
from pyramid.view import view_config
from pyramid.httpexceptions import HTTPFound
from pyramid_mailer import get_mailer
from pyramid_mailer.message import Message


from ringo.lib.sql import DBSession
from ringo.model.base import BaseFactory
from ringo.model.user import User, Profile, Usergroup, USER_GROUP_ID
from formbar.config import Config, load
from formbar.form import Form, Validator

from ringo.views import handle_history
from ringo.lib.helpers import get_path_to_form_config
from ringo.lib.security import login as user_login, request_password_reset, \
    password_reset, activate_user


def is_login_unique(field, data):
    """Validator function as helper for formbar validators"""
    users = DBSession.query(User).filter_by(login=data[field]).all()
    return len(users) == 0


@view_config(route_name='login', renderer='/auth/login.mako')
def login(request):
    handle_history(request)
    _ = request.translate
    config = Config(load(get_path_to_form_config('auth.xml', 'ringo')))
    form_config = config.get_form('loginform')
    form = Form(form_config)
    if request.POST:
        form.validate(request.params.mixed())
        username = form.data.get('login')
        password = form.data.get('pass')
        user = user_login(username, password)
        if user is None:
            msg = _("Login failed!")
            request.session.flash(msg, 'error')
        else:
            msg = _("Login was successfull :)")
            request.session.flash(msg, 'success')
            headers = remember(request, user.id, max_age='86400')
            target_url = request.route_url('home')
            return HTTPFound(location=target_url, headers=headers)
    return {'form': form.render()}


@view_config(route_name='logout', renderer='/auth/logout.mako')
def logout(request):
    handle_history(request)
    _ = request.translate
    target_url = request.route_url('home')
    headers = forget(request)
    msg = _("Logout was successfull :)")
    request.session.flash(msg, 'success')
    return HTTPFound(location=target_url, headers=headers)


@view_config(route_name='register_user',
             renderer='/auth/register_user.mako')
def register_user(request):
    handle_history(request)
    _ = request.translate
    settings = request.registry.settings
    config = Config(load(get_path_to_form_config('auth.xml', 'ringo')))
    form_config = config.get_form('register_user')
    form = Form(form_config)
    # Do extra validation which is not handled by formbar.
    # Is the login unique?
    validator = Validator('login',
                          'There is already a user with this name',
                          is_login_unique)
    form.add_validator(validator)
    if request.POST:
        if form.validate(request.params.mixed()):
            # 1. Create user. Do not activate him. Default role is user.
            ufac = BaseFactory(User)
            pfac = BaseFactory(Profile)
            gfac = BaseFactory(Usergroup)
            user = ufac.create(None)
            # Set login from formdata
            user.login = form.data['login']
            # Encrypt password and save
            pw = hashlib.md5()
            pw.update(form.data['pass'])
            user.password = pw.hexdigest()
            # Deactivate the user. To activate the user needs to confirm
            # with the activation link
            user.activated = False
            atoken = str(uuid.uuid4())
            user.activation_token = atoken
            # Set user group
            group = gfac.load(USER_GROUP_ID)
            user.groups.append(group)
            # Set default user group.
            user.gid = group.id
            DBSession.add(user)
            profile = pfac.create(None)
            profile.email = form.data['email']
            profile.user = user
            DBSession.add(profile)
            # 3. Send confirmation email. The user will be activated
            #    after the user clicks on the confirmation link
            sender = settings['mail.default_sender']
            recipient = profile.email
            subject = _('Confirm user registration for ringo')
            message = _('Please confirm the user registration by clicking'
                        ' on the following link: %s'
                        % request.route_url('confirm_user',
                                            token=atoken))
            _send_mail(request, recipient, sender, subject, message)
            target_url = request.route_url('login')
            headers = forget(request)
            msg = _("User has been created and a confirmation mail was sent"
                    " to the users email adress. Please check your email :)")
            request.session.flash(msg, 'success')
            return HTTPFound(location=target_url, headers=headers)
    return {'form': form.render()}


@view_config(route_name='confirm_user',
             renderer='/auth/confirm_user.mako')
def confirm_user(request):
    handle_history(request)
    _ = request.translate
    success = False
    token = request.matchdict.get('token')
    user = activate_user(token, request.db)
    if user:
        success = True
        msg = _("The user has beed successfull confirmed.")
    else:
        msg = _("The user was not confirmed. Maybe the confirmation"
                " token was not valid?")
    return {'msg': msg, 'success': success}


@view_config(route_name='forgot_password',
             renderer='/auth/forgot_password.mako')
def forgot_password(request):
    handle_history(request)
    _ = request.translate
    settings = request.registry.settings
    config = Config(load(get_path_to_form_config('auth.xml', 'ringo')))
    form_config = config.get_form('forgot_password')
    form = Form(form_config)
    if request.POST:
        if form.validate(request.params.mixed()):
            username = form.data.get('login')
            user = request_password_reset(username, request.db)
            if user:
                # Generate email with the password request token.
                sender = settings['mail.default_sender']
                # TODO: Why profile is a InstrumentedList? For now take
                # the only profile of the user
                recipient = user.profile[0].email
                subject = _('Password reset request for xxx')
                message = "Visit this URL to confirm a password reset: %s" \
                          % request.route_url('reset_password',
                                              token=user.reset_tokens[-1])
                _send_mail(request, recipient, sender, subject, message)
            target_url = request.route_url('login')
            headers = forget(request)
            msg = _("Password reset token has been sent to the users "
                    "email address. Please check your email :)")
            request.session.flash(msg, 'success')
            return HTTPFound(location=target_url, headers=headers)
    return {'form': form.render()}


@view_config(route_name='reset_password',
             renderer='/auth/reset_password.mako')
def reset_password(request):
    handle_history(request)
    _ = request.translate
    settings = request.registry.settings
    success = False
    token = request.matchdict.get('token')
    user, password = password_reset(token, request.db)
    if password:
        success = True
        # Generate email with the password request token.
        sender = settings['mail.default_sender']
        recipient = user.profile[0].email
        subject = _('Password has been resetted')
        message = "Your password has been resetted tp: %s" % password
        _send_mail(request, recipient, sender, subject, message)
        msg = _("Password has been successfull resetted. "
                "The new password was sent to the users email address."
                " Please check your email.")
    else:
        msg = _("Password was not resetted. Maybe the request"
                " token was not valid?")
    return {'msg': msg, 'success': success}


def _send_mail(request, recipient, sender, subject, message):
    """Will send and email with the subject and body to the recipient
    from the sender

    :recipient: The recipients mail address
    :sender: The senders mail address
    :subject: String of subject
    :message: Mails body.
    :returns: True or False

    """
    message = Message(subject=subject,
                      sender=sender,
                      recipients=[recipient],
                      body=message)
    mailer = get_mailer(request)
    mailer.send(message)



#from pyramid import testing
#from ringo.tests import BaseFunctionalTest, BaseUnitTest
#
#
#class TestViews(BaseUnitTest):
    #def test_login_fails_empty(self):
    #    """ Make sure we can't login with empty credentials"""
    #    self.app.post('/auth/login', status=200)

    #def test_login_fails_empty(self):
    #    """ Make sure we can't login with empty credentials"""
    #    from ringo.views.auth import login
    #    self.config.add_route('login', 'auth/login')
    #    request = testing.DummyRequest(post={
    #        'submit': True,
    #    })
    #    view = login(request)
    #    response = view.post()


    #def test_login_succeeds(self):
    #    """ Make sure we can login """
    #    admin = User(username='sontek', password='temp', kind=u'admin')
    #    admin.activated = True
    #    self.session.add(admin)
    #    self.session.flush()

    #    from app.accounts.views import LoginView
    #    self.config.add_route('index', '/')
    #    self.config.add_route('dashboard', '/dashboard')

    #    request = self.get_csrf_request(post={
    #            'submit': True,
    #            'Username': 'sontek',
    #            'Password': 'temp',
    #        })

    #    view = LoginView(request)
    #    response = view.post()

    #    assert response.status_int == 302



"""Add fields to files table

Revision ID: 3c346dfc5edb
Revises: 1403ac1fa61e
Create Date: 2013-08-15 00:48:52.485146

"""

# revision identifiers, used by Alembic.
revision = '3c346dfc5edb'
down_revision = '1403ac1fa61e'

from alembic import op
import sqlalchemy as sa


def upgrade():
    ### commands auto generated by Alembic - please adjust! ###
    op.add_column('files', sa.Column('data', sa.LargeBinary(), nullable=True))
    op.add_column('files', sa.Column('description', sa.Text(), nullable=True))
    op.add_column('files', sa.Column('mime', sa.Text(), nullable=True))
    op.add_column('files', sa.Column('name', sa.Text(), nullable=True))
    op.add_column('files', sa.Column('size', sa.Integer(), nullable=True))
    ### end Alembic commands ###


def downgrade():
    ### commands auto generated by Alembic - please adjust! ###
    op.drop_column('files', 'size')
    op.drop_column('files', 'name')
    op.drop_column('files', 'mime')
    op.drop_column('files', 'description')
    op.drop_column('files', 'data')
    ### end Alembic commands ###



"""ringo-0.5.0

Revision ID: 4d5e42cec5bc
Revises: None
Create Date: 2013-08-14 23:24:36.139171

"""

# revision identifiers, used by Alembic.
revision = '4d5e42cec5bc'
down_revision = None

from alembic import op
import sqlalchemy as sa


def upgrade():
    ### commands auto generated by Alembic - please adjust! ###
    pass
    ### end Alembic commands ###


def downgrade():
    ### commands auto generated by Alembic - please adjust! ###
    pass
    ### end Alembic commands ###



import unittest
from sqlalchemy.orm.exc import NoResultFound
from ringo.model.user import Profile
from ringo.tests import BaseFunctionalTest

class TestProfile(BaseFunctionalTest):

    def test_wrong_url(self):
        self.login('admin', 'secret')
        self.app.get('/profiles/listtest', status=404)

    def test_list(self):
        self.login('admin', 'secret')
        self.app.get('/profiles/list', status=200)

    def test_read(self):
        self.login('admin', 'secret')
        self.app.get('/profiles/read/1', status=200)

    def test_read_unknown(self):
        self.login('admin', 'secret')
        try:
            self.app.get('/profiles/read/1234')
        except NoResultFound:
            return True
        # Must fail!
        self.assertEqual(0,1)

    def test_edit(self):
        self.login('admin', 'secret')
        self.app.get('/profiles/update/1', status=200)

    def test_edit_save_ok(self):
        self.login('admin', 'secret')
        values = {
            "first_name":"Firstname",
            "last_name":"Lastname",
            "address":"",
            "birthday__month":"MM",
            "birthday__day":"DD",
            "birthday__year":"",
            "email":"mail@example.com",
            "url":"",
            "phone":""
        }
        self.app.post('/profiles/update/1', values, status=302)

if __name__ == '__main__':
    unittest.main()



from behave import *


@when(u'opens the about page')
def step_impl(context):
    context.resp = context.app.get('/about')


@when(u'opens the contact page')
def step_impl(context):
    context.resp = context.app.get('/contact')


@when(u'opens the version page')
def step_impl(context):
    context.resp = context.app.get('/version')


@when(u'opens the home page')
def step_impl(context):
    context.resp = context.app.get('/')

def get_modul_path(modul):
    path = []
    if modul == "user":
        path.append("users")
    elif modul == "usergroup":
        path.append("usergroups")
    elif modul == "role":
        path.append("roles")
    elif modul == "profil":
        path.append("profiles")
    elif modul == "modul":
        path.append("modules")
    elif modul == "appointment":
        path.append("appointments")
    elif modul == "file":
        path.append("files")
    return path

@when(u'opens the create page of modul {modul}')
def step_impl(context, modul):
    path = get_modul_path(modul)
    path.append("create")
    context.resp = context.app.get("/%s" % "/".join(path), status="*")

@when(u'opens the read page for item {id} of modul {modul}')
def step_impl(context, id, modul):
    path = get_modul_path(modul)
    path.append("read")
    path.append(str(id))
    context.resp = context.app.get("/%s" % "/".join(path), status="*")

@when(u'opens the edit page for item {id} of modul {modul}')
def step_impl(context, id, modul):
    path = get_modul_path(modul)
    path.append("update")
    path.append(str(id))
    context.resp = context.app.get("/%s" % "/".join(path), status="*")

@when(u'deletes the item {id} of modul {modul}')
def step_impl(context, id, modul):
    path = get_modul_path(modul)
    path.append("delete")
    path.append(str(id))
    context.resp = context.app.get("/%s" % "/".join(path), status="*")



import datetime
from sqlalchemy.ext.declarative import declared_attr

from sqlalchemy import (
    Column,
    Integer,
    DateTime,
    ForeignKey,
)

from sqlalchemy.orm import (
    relationship,
    backref
)


class Meta(object):
    created = Column(DateTime, default=datetime.datetime.utcnow)
    updated = Column(DateTime, default=datetime.datetime.utcnow)


class Owned(object):

    @declared_attr
    def uid(cls):
        return Column(Integer, ForeignKey('users.id'))

    @declared_attr
    def owner(cls):
        return relationship("User",
                            primaryjoin="User.id==%s.uid" % cls.__name__)

    @declared_attr
    def gid(cls):
        return Column(Integer, ForeignKey('usergroups.id'))

    @declared_attr
    def group(cls):
        return relationship("Usergroup",
                            primaryjoin="Usergroup.id==%s.gid" % cls.__name__)


class Nested(object):
    """Mixin to make nested (self-reference) Items possible. Each item
    can have a parent item and many children. The class will add two
    relation attribute to the inheriting class. The parent item is
    available under the *parent* attribute. The children items are
    available under the *children* attribute."""

    @declared_attr
    def parent_id(cls):
        name = "%s.id" % cls.__tablename__
        return Column(Integer, ForeignKey(name))

    @declared_attr
    def children(cls):
        name = cls.__name__
        join = "%s.id==%s.parent_id" % (name, name)
        return relationship(name,
                            primaryjoin=join,
                            backref=backref('parent', remote_side=[cls.id]))



import logging
import sqlalchemy as sa
from ringo.model import Base
from ringo.model.user import BaseItem

log = logging.getLogger(__name__)


class ActionItem(BaseItem, Base):
    __tablename__ = 'actions'
    _modul_id = 2
    id = sa.Column(sa.Integer, primary_key=True)
    mid = sa.Column(sa.Integer, sa.ForeignKey('modules.id'))
    name = sa.Column(sa.Text, nullable=False)
    url = sa.Column(sa.Text, nullable=False)
    icon = sa.Column(sa.Text, nullable=False)
    description = sa.Column(sa.Text)

    def __unicode__(self):
        return u"%s (%s/%s)" % (self.name, self.modul, self.url)


class ModulItem(BaseItem, Base):
    __tablename__ = 'modules'
    _modul_id = 1
    id = sa.Column(sa.Integer, primary_key=True)
    name = sa.Column(sa.Text, unique=True, nullable=False)
    clazzpath = sa.Column(sa.Text, unique=True, nullable=False)
    label = sa.Column(sa.Text, nullable=False)
    label_plural = sa.Column(sa.Text, nullable=False)
    description = sa.Column(sa.Text)
    str_repr = sa.Column(sa.Text)
    display = sa.Column(sa.Text)

    actions = sa.orm.relationship("ActionItem", backref="modul")

    _sql_joined_relations = ['actions']

    def __unicode__(self):
        return self.name

    def get_label(self, plural=False):
        if plural:
            return self.label_plural
        return self.label

    def has_action(self, url):
        """Will return True if the modul has a ActionItem configured
        with given url. Else false."""
        for action in self.actions:
            if action.url == url:
                return True
        return False


def _create_default_actions(dbsession, ignore=[]):
    # TODO: Translate the name of the Action (torsten) <2013-07-10 09:32> 
    a0 = ActionItem(name="List", url="list", icon="icon-list-alt")
    a1 = ActionItem(name="Create", url="create", icon=" icon-plus")
    a2 = ActionItem(name="Read", url="read/{id}", icon="icon-eye-open")
    a3 = ActionItem(name="Update", url="update/{id}", icon="icon-edit")
    a4 = ActionItem(name="Delete", url="delete/{id}", icon="icon-trash")
    actions = []
    if not "list" in ignore:
        dbsession.add(a0)
        actions.append(a0)
    if not "create" in ignore:
        dbsession.add(a1)
        actions.append(a1)
    if not "read" in ignore:
        dbsession.add(a2)
        actions.append(a2)
    if not "update" in ignore:
        dbsession.add(a3)
        actions.append(a3)
    if not "delete" in ignore:
        dbsession.add(a4)
        actions.append(a4)
    return actions


def init_model(dbsession):
    """Will setup the initial model for the usermanagement. This
    includes creating users, usergroups  roles and permissions.

    :dbsession: Database session to which the items will be added.
    :returns: None

    """

    # ID 1
    modul = ModulItem(name='modules')
    modul.clazzpath = "ringo.model.modul.ModulItem"
    modul.label = "Modul"
    modul.label_plural = "Modules"
    modul.display = "admin-menu"
    modul.actions.extend(_create_default_actions(dbsession, ignore=['create', 'delete']))
    dbsession.add(modul)
    # ID 2
    modul = ModulItem(name='actions')
    modul.clazzpath = "ringo.model.modul.ActionItem"
    modul.label = "Action"
    modul.label_plural = "Actions"
    modul.display = "admin-menu"
    # Actions modul is not meant to be callable from anybody (There is
    # now view at all). So do not add any actions here.
    # modul.actions.extend(_create_default_actions(dbsession))
    dbsession.add(modul)
    # ID 3
    modul = ModulItem(name='users')
    modul.clazzpath = "ringo.model.user.User"
    modul.display = "admin-menu"
    modul.label = "User"
    modul.label_plural = "Users"
    modul.actions.extend(_create_default_actions(dbsession))
    dbsession.add(modul)
    # ID 4
    modul = ModulItem(name='usergroups')
    modul.clazzpath = "ringo.model.user.Usergroup"
    modul.label = "Usergroup"
    modul.label_plural = "Usergroups"
    modul.display = "admin-menu"
    modul.actions.extend(_create_default_actions(dbsession))
    dbsession.add(modul)
    # ID 5
    modul = ModulItem(name='roles')
    modul.clazzpath = "ringo.model.user.Role"
    modul.label = "Role"
    modul.label_plural = "Roles"
    modul.display = "admin-menu"
    modul.actions.extend(_create_default_actions(dbsession))
    dbsession.add(modul)
    # ID 6
    modul = ModulItem(name='profiles')
    modul.clazzpath = "ringo.model.user.Profile"
    modul.label = "Profile"
    modul.label_plural = "Profiles"
    modul.display = "admin-menu"
    modul.actions.extend(_create_default_actions(dbsession, ignore=['create', 'delete']))
    dbsession.add(modul)



import logging
import json
from mako.lookup import TemplateLookup
from formbar.renderer import (
    DropdownFieldRenderer as FormbarDropdown,
    SelectionFieldRenderer as FormbarSelectionField
)
from ringo import template_dir
from ringo.lib.helpers import (
    get_saved_searches,
    get_path_to_overview_config,
)
from ringo.model.base import BaseItem

template_lookup = TemplateLookup(directories=[template_dir],
                                 module_directory='/tmp/ringo_modules')

log = logging.getLogger(__name__)


def _load_overview_config(clazz):
        """Return a datastructure representing the overview
        configuration. The configuration is loaded from a JSON
        configuration file under /view/overviews relative to the
        application root. If no configuration can be found return
        None."""
        cfile = "%s.json" % clazz.__tablename__
        # Try to load the configuration for the overview first.
        config = None
        try:
            config = open(get_path_to_overview_config(cfile), "r")
        except IOError:
            try:
                config = open(get_path_to_overview_config(cfile, 'ringo'), "r")
            except IOError:
                log.warning('Can not load overview configuration for "%s" '
                            'Configuring the overview now based on the form '
                            'configuration' % clazz)

        if config:
            return json.load(config)
        return None


def _form2overview(formconfig):
    """Converts a the form configuration of the given clazz into a
    datastructure representing the overview configuration."""
    fields = []
    for key, field in formconfig.get_fields().iteritems():
        fields.append({'name': field.name, 'label': field.label})
    return {'overview': {'columns': fields}}


class TableConfig:
    """The TableConfig clazz provides an interface for configuring the
    overview of the the given clazz. The configuration includes

    * Enabled features of the overview
    * Which fields should be enabled in the overview
    * Labeling of the fields
    * Rendering of the fields
    * Layout of the table (column width etc.)

    The configuration of the overview is loaded from a an JSON
    configuration file which is located under view/overviews relative to
    the application root.

    The configuration has the following form and options::

        {
            "overview": {
                "columns": [
                    {
                        "name": "fieldname",
                        "label": "Label",
                        "width": "width",
                        "expand": true
                    }
                ]
                "settings": {
                    "default-sorting": "name"
                }
            }
        }

    The configuration can have configurations for more than one table
    configuration. The default name of a configuration is *overview*.
    The next sections defines the *columns* of the table. The columns
    are rendered in the order they are defined. Each column can have the
    following options:

    * *name*: The name of the attribute within clazz which should be
      listed here.
    * *label*: The label of the field.
    * *width*: The width of the field. If not units are given the pixel
      are assumed.
    * *expand*: The expand option is used to expand the referneces
      values in selections into the literal value of the corrispondig
      option. Note that this option is only usefull for selection fields
      in *formbar* which do not have a real relation attached. In all
      other cases the reference values are expanded automatically.

    If no configuration file can be found, then add all fields
    configured in the form configuration to the overview.
    """

    def __init__(self, clazz, name):
        """Will initialize the configuration of the overview for the
        clazz.

        :clazz: The clazz for which the overview configuration should be
        loaded.
        :name: name of the configuration. Defaults to the
        "overview" configuration.
        """
        self.clazz = clazz
        self.name = name or "overview"
        config = _load_overview_config(clazz)
        if config:
            self.config = config
        else:
            form_config = self.get_form_config()
            self.config = _form2overview(form_config)

    def get_columns(self):
        """Return a list of configured columns within the configuration.
        Each colum is a dictionary containing the one or more available
        conifguration attributes."""
        cols = []
        config = self.config.get(self.name)
        for col in config.get('columns'):
            cols.append(col)
        return cols

    def get_form_config(self, name="create"):
        return self.clazz.get_form_config(name)

    def get_default_sort_column(self):
        """Returns the name of the attribute of the clazz which is
        marked as field for default sorting in the *settings* section of
        the configuration. If no default sorting is configured then
        return the name of the attribute in the first column which is
        configured in the table"""
        settings = self.config.get('settings')
        if settings:
            def_sort = settings.get('default-sorting')
            if def_sort:
                return def_sort
        return self.get_columns()[0].get('name')


class Renderer(object):
    """Baseclass for all renderers"""

    def __init__(self):
        """Initialize renderer"""
        pass

    def render(self):
        """Initialize renderer"""
        pass


###########################################################################
#                         Renderers for overviews                         #
###########################################################################

class ListRenderer(Renderer):
    """Docstring for ListRenderer """

    def __init__(self, listing):
        """@todo: to be defined """
        Renderer.__init__(self)
        self.listing = listing
        self.template = template_lookup.get_template("internal/list.mako")

    def render(self, request):
        """Initialize renderer"""
        if len(self.listing.search_filter) > 0:
            search = self.listing.search_filter[-1][0]
            search_field = self.listing.search_filter[-1][1]
        else:
            search = ""
            search_field = ""
        values = {'items': self.listing.items,
                  'clazz': self.listing.clazz,
                  'listing': self.listing,
                  'request': request,
                  '_': request.translate,
                  'enable_bundled_actions': False,
                  'search': search,
                  'search_field': search_field,
                  'saved_searches': get_saved_searches(request,
                                                       self.listing.clazz.__tablename__),
                  'tableconfig': self.listing.clazz.get_table_config()}
        return self.template.render(**values)


class DTListRenderer(Renderer):
    """Docstring for a ListRenderer using the DataTables Jquery Plugin"""
    def __init__(self, listing):
        Renderer.__init__(self)
        self.listing = listing
        self.template = template_lookup.get_template("internal/dtlist.mako")

    def render(self, request):
        """Initialize renderer"""
        values = {'items': self.listing.items,
                  'listing': self.listing,
                  'request': request,
                  '_': request.translate,
                  'tableconfig': self.listing.clazz.get_table_config()}
        return self.template.render(**values)


###########################################################################
#                         Renderers for dialogs                           #
###########################################################################


class DialogRenderer(Renderer):
    """Renderer for Dialogs"""

    def __init__(self, request, item, action, title=None, body=None):
        """Renders a renderered dialog for the requested action on the
        given item. If not header or body is provided the dialog will
        have a default message.

        :request: The current request
        :item: The item for which the item should be confirmed.
        :action: The actions which must be confirmend.
        :header: Custom text for the header of the dialog
        :body: Custom text for the body of the dialog.

        """
        Renderer.__init__(self)

        self._request = request
        self._item = item
        self._action = action
        self._title = title
        self._body = body


class ConfirmDialogRenderer(DialogRenderer):
    """Docstring for ConfirmDialogRenderer """

    def __init__(self, request, item, action, title=None, body=None):
        """@todo: to be defined """
        DialogRenderer.__init__(self, request, item, action, title, body)
        self.template = template_lookup.get_template("internal/confirm.mako")

    def render(self):
        _ = self._request.translate
        mapping = {'Action': self._action.capitalize()}
        values = {}
        values['icon'] = self._request.static_url(
            'ringo:static/images/icons/32x32/dialog-warning.png')
        values['header'] = _("Confirm ${Action}", mapping=mapping)
        values['body'] = self._render_body()
        values['action'] = self._action.capitalize()
        values['ok_url'] = self._request.current_route_url()
        values['_'] = self._request.translate
        values['cancel_url'] = self._request.referrer
        return self.template.render(**values)

    def _render_body(self):
        out = []
        _ = self._request.translate
        item_label = self._item.get_item_modul().get_label()
        mapping = {'action': self._action, 'item': item_label,
                   'Action': self._action.capitalize()}
        out.append(_("Do you really want to ${action}"
                     " the following ${item} items?",
                     mapping=mapping))
        out.append("<br>")
        out.append("<ol>")
        if isinstance(self._item, list):
            items = self._item
        else:
            items = [self._item]
        for item in items:
            out.append("<li>")
            out.append(unicode(item))
            out.append("</li>")
        out.append("</ol>")
        out.append(_('Please press "${Action}" to ${action} the item.'
                     ' Press "Cancel" to cancel the action.',
                     mapping=mapping))

        return "".join(out)


class ErrorDialogRenderer(DialogRenderer):
    """Docstring for ErrorDialogRenderer """

    def __init__(self, request, title, body):
        """@todo: to be defined """
        DialogRenderer.__init__(self, request, None, None, title, body)
        self.template = template_lookup.get_template("internal/error.mako")

    def render(self):
        values = {}
        values['icon'] = self._request.static_url(
            'ringo:static/images/icons/32x32/dialog-error.png')
        values['header'] = self._title
        values['body'] = self._render_body()
        history = self._request.session.get('history')
        if history:
            values['ok_url'] = self._request.session['history'].pop()
        else:
            values['ok_url'] = self._request.route_url('home')
        return self.template.render(**values)

    def _render_body(self):
        out = []
        out.append(self._body)
        return "".join(out)


###########################################################################
#                         Renderers for form elements                     #
###########################################################################


class DropdownFieldRenderer(FormbarDropdown):
    """Ringo specific DropdownFieldRenderer. This renderer add a small
    link next to the label to make it possible to jump to the selected
    item in the dropdown list."""

    def __init__(self, field, translate):
        """@todo: to be defined"""
        FormbarDropdown.__init__(self, field, translate)

    def _render_link(self):
        html = []
        form = self._field._form
        try:
            item = getattr(form._item, self._field.name)
        except AttributeError:
            # Can happen when designing forms an the model of the item
            # is not yet configured.
            log.warning("Missing %s attribute in %s" % (form._item,
                                                        self._field.name))
            item = None
        if isinstance(item, BaseItem):
            url = "/%s/read/%s" % (item.__tablename__, item.id)
            html.append('<a href="%s">&nbsp;[%s]</a>' % (url, item))
        return "".join(html)

    def _render_label(self):
        html = []
        html.append(FormbarDropdown._render_label(self))
        html.append(self._render_link())
        return "".join(html)


class ListingFieldRenderer(FormbarSelectionField):

    def __init__(self, field, translate):
        FormbarSelectionField.__init__(self, field, translate)
        self.all_items = self._get_all_items()
        self.template = template_lookup.get_template("internal/listfield.mako")

    def _get_all_items(self):
        clazz = self._field._get_sa_mapped_class()
        return clazz.get_item_list(self._field._form._dbsession)

    def _get_selected_items(self):
        try:
            items = getattr(self._field._form._item, self._field.name)
        except AttributeError:
            # Can happen when designing forms an the model of the item
            # is not yet configured.
            log.warning("Missing %s attribute in %s" % (self._field._form._item,
                                                        self._field.name))
            items = []
        return items

    def render(self):
        """Initialize renderer"""
        config = self._field._config.renderer.config
        if self._field.is_readonly():
            items = self._get_selected_items()
        else:
            items = self.all_items.items
        values = {'items': items,
                  'field': self._field,
                  'clazz': self._field._get_sa_mapped_class(),
                  'request': self._field._form._request,
                  '_': self._field._form._translate,
                  'tableconfig': self.all_items.clazz.get_table_config(config)}
        return self.template.render(**values)



"""0.6.0

Revision ID: 2a484092ec79
Revises: 3c346dfc5edb
Create Date: 2013-08-19 22:51:49.746128

"""

INSERTS = """
insert into modules ("id", "name", "clazzpath", "label", "label_plural", "description", "str_repr", "display") values ('8', 'files', 'ringo.model.file.File', 'File', 'Files', NULL, NULL, 'header-menu');

"""

# revision identifiers, used by Alembic.
revision = '2a484092ec79'
down_revision = '3c346dfc5edb'

from alembic import op
import sqlalchemy as sa


def upgrade():
    op.execute(INSERTS)
    pass


def downgrade():
    op.execute("DELETE from modules where id = 8")



"""Added file table

Revision ID: 1403ac1fa61e
Revises: 4d5e42cec5bc
Create Date: 2013-08-14 23:29:09.622395

"""

# revision identifiers, used by Alembic.
revision = '1403ac1fa61e'
down_revision = '4d5e42cec5bc'

from alembic import op
import sqlalchemy as sa


def upgrade():
    ### commands auto generated by Alembic - please adjust! ###
    op.create_table('files',
        sa.Column('id', sa.Integer(), nullable=False),
        sa.Column('uid', sa.Integer(), nullable=True),
        sa.Column('gid', sa.Integer(), nullable=True),
        sa.ForeignKeyConstraint(['gid'], ['usergroups.id'], ),
        sa.ForeignKeyConstraint(['uid'], ['users.id'], ),
        sa.PrimaryKeyConstraint('id')
    )
    op.execute("INSERT into modules (id, name, clazzpath, label, label_plural, display) VALUES (8, 'files', 'ringo.model.file.File', 'File', 'Files', 'header-menu')")
    op.execute("insert into actions (id, mid, name, url, icon, description) values ('27', '8', 'List', 'list', 'icon-list-alt', NULL)")
    op.execute("insert into actions (id, mid, name, url, icon, description) values ('28', '8', 'Create', 'create', ' icon-plus', NULL)")
    op.execute("insert into actions (id, mid, name, url, icon, description) values ('29', '8', 'Read', 'read/{id}', 'icon-eye-open', NULL)")
    op.execute("insert into actions (id, mid, name, url, icon, description) values ('30', '8', 'Update', 'update/{id}', 'icon-edit', NULL)")
    op.execute("insert into actions (id, mid, name, url, icon, description) values ('31', '8', 'Delete', 'delete/{id}', 'icon-trash', NULL)")
    ### end Alembic commands ###


def downgrade():
    ### commands auto generated by Alembic - please adjust! ###
    op.drop_table('files')
    op.execute("DELETE from modules where id = 8")
    op.execute("DELETE from actions where id = 27")
    op.execute("DELETE from actions where id = 28")
    op.execute("DELETE from actions where id = 29")
    op.execute("DELETE from actions where id = 30")
    op.execute("DELETE from actions where id = 31")
    ### end Alembic commands ###



import unittest
from ringo.tests import BaseFunctionalTest
from ringo.tests.functional.test_users import TestUser


class TestAuth(BaseFunctionalTest):

    def test_403(self):
        self.app.get('/users/list', status=403)

    def test_login(self):
        self.app.get('/auth/login', status=200)

    def test_login_ok(self):
        self.logout()
        values = {'login': 'admin', 'pass': 'secret'}
        self.app.post('/auth/login', values, status=302)

    def test_login_fail(self):
        self.logout()
        values = {'login': 'admin', 'pass': 'secret2'}
        self.app.post('/auth/login', values, status=200)

    def test_logout(self):
        self.app.get('/auth/logout', status=302)

    def test_register(self):
        self.app.get('/auth/register_user', status=200)

    def test_register_fail(self):
        values = {
            'login': 'admin',
            'pass': '12345678',
            'pass2': '12345678',
            'email': 'test@example.com'
        }
        self.app.post('/auth/register_user', values, status=200)

    def test_register_ok(self):
        values = {
            'login': 'newuser',
            'pass': '12345678',
            'pass2': '12345678',
            'email': 'test@example.com'
        }
        self.app.post('/auth/register_user', values, status=302)

    def test_forgot(self):
        self.app.get('/auth/forgot_password', status=200)

    def test_forgot_fail(self):
        values = {
            'login': 'xxx'
        }
        # Note that the response is the same as on success here as we do
        # not want to provide any information if ther is a user with
        # this login.
        response = self.app.post('/auth/forgot_password', values, status=302).follow()
        assert 'Password reset token has been sent' in response

    def test_forgot_ok(self):
        values = {
            'login': 'admin'
        }
        response = self.app.post('/auth/forgot_password', values, status=302).follow()
        assert 'Password reset token has been sent' in response

    def test_confirm_ok(self):
        response = self.app.get('/auth/confirm_user/0e3cc848-22f6-4ff7-a562-12baf3037439', status=200)
        assert 'The user has beed successfull confirmed.' in response

    def test_confirm_fail(self):
        response = self.app.get('/auth/confirm_user/xxx', status=200)
        assert 'The user was not confirmed' in response

    # Token is already reseted on confirmation. So this test will fail.
    # def test_reset_ok(self):
    #    self.login('admin', 'secret')
    #    values = {
    #            'login': 'admin',
    #            'groups': '1',
    #            'gid': '',
    #            'activated': 'True',
    #            'activation_token': '0e3cc848-22f6-4ff7-a562-12baf3037439'
    #    }
    #    self.app.post('/users/update/1', values, status=302)
    #    response = self.app.get('/auth/reset_password/0e3cc848-22f6-4ff7-a562-12baf3037439', status=200)
    #    assert 'Password has been successfull resetted' in response

    def test_reset_fail(self):
        response = self.app.get('/auth/reset_password/xxx', status=200)
        assert 'Password was not resetted' in response

if __name__ == '__main__':
    unittest.main()



import unittest
import re
from sqlalchemy.orm.exc import NoResultFound
from ringo.model.modul import ModulItem
from ringo.tests import BaseFunctionalTest

class TestModulItem(BaseFunctionalTest):

    def test_wrong_url(self):
        self.login('admin', 'secret')
        self.app.get('/modules/listtest', status=404)

    def test_list(self):
        self.login('admin', 'secret')
        self.app.get('/modules/list', status=200)

    def test_list_search(self):
        self.login('admin', 'secret')
        values = {
            'search': 'modu',
            'field': 'name',
            'form': 'search'
        }
        self.app.post('/modules/list', values, status=200)

    def test_list_search_empty(self):
        self.login('admin', 'secret')
        values1 = {
            'search': 'modu',
            'field': 'name',
            'form': 'search'
        }
        values2 = {
            'search': '',
            'field': 'name',
            'form': 'search'
        }
        self.app.post('/modules/list', values1, status=200)
        self.app.post('/modules/list', values2, status=200)

    def test_list_search_no_result(self):
        self.login('admin', 'secret')
        values1 = {
            'search': 'modu',
            'field': '',
            'form': 'search'
        }
        values2 = {
            'search': 'xxxxxx',
            'field': 'name',
            'form': 'search'
        }
        self.app.post('/modules/list', values1, status=200)
        self.app.post('/modules/list', values2, status=200)


    def test_read(self):
        self.login('admin', 'secret')
        self.app.get('/modules/read/1', status=200)

    def test_read_unknown(self):
        self.login('admin', 'secret')
        try:
            self.app.get('/modules/read/1234')
        except NoResultFound:
            return True
        # Must fail!
        self.assertEqual(0,1)

    def test_edit(self):
        self.login('admin', 'secret')
        self.app.get('/modules/update/1', status=200)

    # TODO: Not sure how to craft a POST request. This request seems to
    # loose actions which causes routing errors later.
    #def test_edit_save_ok(self):
    #    self.login('admin', 'secret')
    #    values = {
    #        "name":"modules",
    #        "label":"Modul",
    #        "label_plural":"Modules",
    #        "description":"",
    #        "str_repr":"",
    #        "actions":["1", "2", "3"],
    #    }
    #    self.app.post('/modules/update/1', values, status=302)

if __name__ == '__main__':
    unittest.main()



import logging
from sqlalchemy.orm.exc import NoResultFound
from pyramid.security import (
    Allow,
    ALL_PERMISSIONS
)
from ringo.lib.security import get_roles

log = logging.getLogger(__name__)


def get_resource_factory(clazz):
    """Dynamically create a Resource factory"""
    name = clazz.__name__
    factory = type(name, (RessourceFactory, ), {'__model__': clazz})
    return factory


class RessourceFactory(object):

    # Users with the admin role should be allowed to to all actions
    __default_acl__ = [(Allow, 'role:admin', ALL_PERMISSIONS)]

    def __init__(self, request):
        # Reset ACL
        self.__acl__ = []
        self.item = None

        item_id = request.matchdict.get('id')
        if item_id:
            self.item = self._load_item(item_id)
        self._set_acl(request)

    def _load_item(self, id):
        # TODO: Make loading items more robust in RessourceFactory. This
        # is needed in case that a Ressource is build for a clazz which
        # is not related to the  the current request which has an id. In
        # this case loading the item might fail if there is no item with
        # the id in the request.
        #
        # If think this code might cause problems as trying to build a
        # build a Ressource with an independet request smells a
        # littlebit. Better add a flag to to ignore the id in the
        # request in such cases.
        try:
            factory = self.__model__.get_item_factory()
            return factory.load(id)
        except NoResultFound:
            return None

    def _set_default_acl(self):
        self.__acl__.extend(self.__default_acl__)

    def _get_role_permissions(self, request):
        """Will return a list the assigned permissions for the current
        user in the request.

        If the a role of a user is an administrational role thean all
        permissions to this role are added to the returned list without
        checking any ownership. 
        If it is not an administrational role, the "list" and "create"
        actions are added in all cases. Item related action like "edit",
        "delete" or "read" are only added if the current user is
        actually the owner.

        :request: current request
        :returns: List of Action items (Permissions)

        """
        permissions = []
        user = request.user
        if user is None:
            return []
        for role in get_roles(user):
            log.debug("Setting permissions for role %s" % role)
            for perm in role.permissions:
                if perm.modul.id == self.__model__._modul_id:
                    permission = None
                    if role.admin == True:
                        # Set administrational role based permissions. For
                        # administrational roles no ownership is
                        # checked an the user will get all permissions which are
                        # assigned to the role.
                        permission = (Allow, 'role:%s' % role, perm.name.lower())
                    else:
                        # Set user roles based permission. In contrast to the
                        # administrational roles the user will only get the permissions
                        # assigned to the role if he is the owner of the
                        # item or if the action is "list" of "create".
                        if perm.name.lower() in ['list', 'create']:
                            permission = (Allow, 'role:%s' % role, perm.name.lower())
                        elif self.item:
                            permission = (Allow, 'uid:%s' % self.item.uid, perm.name.lower())
                    if permission:
                        log.debug("adding new permission: %s" % str(permission))
                        permissions.append(permission)
        return permissions

    def _set_acl(self, request):
        """Will dynamically set the __acl__ for this ressource for the current request.

        :request: current request
        :returns: None

        """
        log.debug("Setting ACL for user %s" % request.user)
        self._set_default_acl()
        self.__acl__.extend(self._get_role_permissions(request))


class Resource(object):
    """Ressource element"""

    __name__ = ""
    __parent__ = None

    # Default ACL
    __acl__ = [(Allow, 'role:admin', ('create', 'read', 'update',
                                      'delete', 'list'))]

    def __init__(self, title=None, model=None):
        self._childs = {}
        self._title = title
        self._model = title

    def add_child(self, url, resource):
        resource.__name__ = url
        resource.__parent__ = self
        self._childs[url] = resource

    def __getitem__(self, name):
        log.debug('Searching for resource "%s" in %s' % (name, self._childs))
        try:
            resource = self._childs[name]
            return resource
        except KeyError:
            raise


class Root(Resource):
    def __init__(self, request):
        super(Root, self).__init__("Root")
        self.request = request



import unittest
from ringo.tests import BaseFunctionalTest

class TestRole(BaseFunctionalTest):

    def test_about(self):
        self.app.get('/about', status=200)

    def test_contact(self):
        self.app.get('/contact', status=200)

    def test_version(self):
        self.app.get('/version', status=200)

    def test_home(self):
        self.app.get('/', status=200)

if __name__ == '__main__':
    unittest.main()
